@@ -3,7 +3,7 @@ LaTeX allows you to define your own commands using `\newcommand`.
 This can be a useful feature for many scenarios, but it complicates the source.
 Thus, this file implements some logic to substitute these usages by their definition.
 """
-
+import logging
 import re
 import typing
 
@@ -27,7 +27,7 @@ class NewCommandDefinition:
         :param command: The actual command definition.
         """
         if not isinstance(command, TraceableString):
-            print("WARNING: Command is not traceable!")
+            logging.getLogger("flachtex").warning("Command is not traceable!")
             command = TraceableString(str(command), None)
         self.name = name
         self.num_parameters = num_parameters
@@ -67,9 +67,13 @@ class NewCommandSubstitution(SubstitutionRule):
     Currently, default parameters are not supported.
     """
 
-    def __init__(self):
+    def __init__(self, space_substitution: bool = True):
+        """
+        :param space_substitution: Try to simulate the missing space of parameterless commands.
+        """
         self._commands = {}
         self._command_finder = CommandFinder()
+        self._space_sub = space_substitution
 
     def new_command(self, definition: NewCommandDefinition) -> None:
         """
@@ -81,12 +85,11 @@ class NewCommandSubstitution(SubstitutionRule):
         if name[0] == "\\":
             name = name[1:]
         if name in self._commands:
-            print(
-                f"%WARNING: Multiple definitions of command '{name}'."
-                f" Substitution may be buggy."
+            logging.getLogger("flachtex").warning(
+                f"Multiple definitions of command '{name}'. Substitution may be buggy."
             )
         self._commands[name] = definition
-        print("%LOG: Detected", definition)
+        logging.getLogger("flachtex").info(f"Detected {definition}")
         self._command_finder.add_command(name, definition.num_parameters)
 
     def _get_substitution(
@@ -94,8 +97,7 @@ class NewCommandSubstitution(SubstitutionRule):
     ) -> TraceableString:
         for i, p in enumerate(parameters):
             offset = 0
-            for match in re.findall(f"#{i}([^0-9]|$)", str(command), re.MULTILINE):
-                print(type(command))
+            for match in re.finditer(f"#{i}([^0-9]|$)", str(command), re.MULTILINE):
                 command = (
                         command[: match.start() + offset]
                         + p
@@ -110,9 +112,12 @@ class NewCommandSubstitution(SubstitutionRule):
             parameters = [content[p[0]: p[1]] for p in match.parameters]
             sub = self._get_substitution(definition.command, parameters)
             end = match.end
-            if definition.num_parameters == 0:  # space after command is just a separator
-                while content[end] == " ":
-                    end += 1
-                if end != match.end:
-                    sub += TraceableString("{}", None)  # add non-space seperator
+            if self._space_sub and  definition.num_parameters == 0:
+                # The usage of a command like "\\cmd bla" is actually equivalent to
+                # "\\cmd{}bla". This function tries to simulate this.
+                if not str(sub).strip().endswith("\\xspace"):
+                    while content[end] == " ":
+                        end += 1
+                    if end != match.end:
+                        sub += TraceableString("{}", None)  # add non-space seperator
             yield Substitution(match.start, end, sub)
